#include <nemo/byte.h>
#include <nemo/error.h>

#include "djbio.h"

static int oneread(register ssize_t (*op)(int,void*,size_t), register int fd, register void *buf, register unsigned int len)
{
  register ssize_t r;

  for (;;) {
    r = op(fd, buf, len);
    if (r < 0) {
      if (errno == error_intr) continue;
    }
    return (int)r;
  }
}

static int getthis(register djbio *iop, register void *buf, register unsigned int len)
{
  register unsigned int r;
  register int q;

  r = iop->p;
  q = (int)r - (int)len;
  if (q > 0) {
    r = len;
    iop->p = (unsigned int)q;
  }
  else {
    iop->p = 0;
  }
  byte_copy(buf, r, iop->x + iop->n);
  iop->n += r;
  return (int)r;
}

int djbio_feed(register djbio *iop)
{
  register int r;
  register int q;

  if (iop->p) return (int)iop->p;
  q = (int)iop->n;
  r = oneread(iop->read_op, iop->fd, iop->x, (unsigned int)q);
  if (r <= 0) return r;
  iop->p = (unsigned int)r;
  q -= r;
  iop->n = (unsigned int)q;
  if (q > 0) {
    /* damn,  gotta shift */
    byte_copyr(iop->x + q, (unsigned int)r, iop->x);
  }
  return r;
}

int djbio_bget(register djbio *iop, register void *buf, register unsigned int len)
{
  register int r;
  register unsigned int i;

  if (iop->p > 0) {
    return getthis(iop, buf, len);
  }
  i = iop->n;
  if (i <= len) {
    return oneread(iop->read_op, iop->fd, buf, i);
  }
  r = djbio_feed(iop);
  return (r <= 0) ? r : getthis(iop, buf, len);
}

int djbio_get(register djbio *iop, register void *buf, register unsigned int len)
{
  register int r;

  if (iop->p > 0) {
    return getthis(iop, buf, len);
  }
  if (iop->n <= len) {
    return oneread(iop->read_op, iop->fd, buf, len);
  }
  r = djbio_feed(iop);
  return (r <= 0) ? r : getthis(iop, buf, len);
}
